@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,23 +21,21 @@
 
 /**
  * Class containing methods for operations with items.
- *
- * @package API
  */
 class CItem extends CItemGeneral {
 
 	protected $tableName = 'items';
 	protected $tableAlias = 'i';
-	protected $sortColumns = array('itemid', 'name', 'key_', 'delay', 'history', 'trends', 'type', 'status');
+	protected $sortColumns = ['itemid', 'name', 'key_', 'delay', 'history', 'trends', 'type', 'status'];
 
 	public function __construct() {
 		parent::__construct();
 
-		$this->errorMessages = array_merge($this->errorMessages, array(
+		$this->errorMessages = array_merge($this->errorMessages, [
 			self::ERROR_EXISTS_TEMPLATE => _('Item "%1$s" already exists on "%2$s", inherited from another template.'),
 			self::ERROR_EXISTS => _('Item "%1$s" already exists on "%2$s".'),
 			self::ERROR_INVALID_KEY => _('Invalid key "%1$s" for item "%2$s" on "%3$s": %4$s.')
-		));
+		]);
 	}
 
 	/**
@@ -59,21 +57,19 @@ class CItem extends CItemGeneral {
 	 *
 	 * @return array|int item data as array or false if error
 	 */
-	public function get($options = array()) {
-		$result = array();
-		$userType = self::$userData['type'];
-		$userid = self::$userData['userid'];
+	public function get($options = []) {
+		$result = [];
 
-		$sqlParts = array(
-			'select'	=> array('items' => 'i.itemid'),
-			'from'		=> array('items' => 'items i'),
-			'where'		=> array('webtype' => 'i.type<>'.ITEM_TYPE_HTTPTEST, 'flags' => 'i.flags IN ('.ZBX_FLAG_DISCOVERY_NORMAL.','.ZBX_FLAG_DISCOVERY_CREATED.')'),
-			'group'		=> array(),
-			'order'		=> array(),
+		$sqlParts = [
+			'select'	=> ['items' => 'i.itemid'],
+			'from'		=> ['items' => 'items i'],
+			'where'		=> ['webtype' => 'i.type<>'.ITEM_TYPE_HTTPTEST, 'flags' => 'i.flags IN ('.ZBX_FLAG_DISCOVERY_NORMAL.','.ZBX_FLAG_DISCOVERY_CREATED.')'],
+			'group'		=> [],
+			'order'		=> [],
 			'limit'		=> null
-		);
+		];
 
-		$defOptions = array(
+		$defOptions = [
 			'groupids'					=> null,
 			'templateids'				=> null,
 			'hostids'					=> null,
@@ -87,7 +83,7 @@ class CItem extends CItemGeneral {
 			'inherited'					=> null,
 			'templated'					=> null,
 			'monitored'					=> null,
-			'editable'					=> null,
+			'editable'					=> false,
 			'nopermissions'				=> null,
 			'group'						=> null,
 			'host'						=> null,
@@ -97,8 +93,8 @@ class CItem extends CItemGeneral {
 			'filter'					=> null,
 			'search'					=> null,
 			'searchByAny'				=> null,
-			'startSearch'				=> null,
-			'excludeSearch'				=> null,
+			'startSearch'				=> false,
+			'excludeSearch'				=> false,
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
@@ -109,21 +105,21 @@ class CItem extends CItemGeneral {
 			'selectApplications'		=> null,
 			'selectDiscoveryRule'		=> null,
 			'selectItemDiscovery'		=> null,
-			'countOutput'				=> null,
-			'groupCount'				=> null,
-			'preservekeys'				=> null,
+			'selectPreprocessing'		=> null,
+			'countOutput'				=> false,
+			'groupCount'				=> false,
+			'preservekeys'				=> false,
 			'sortfield'					=> '',
 			'sortorder'					=> '',
 			'limit'						=> null,
 			'limitSelects'				=> null
-		);
+		];
 		$options = zbx_array_merge($defOptions, $options);
 
 		// editable + permission check
-		if ($userType != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
 			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
-
-			$userGroups = getUserGroupsByUserId($userid);
+			$userGroups = getUserGroupsByUserId(self::$userData['userid']);
 
 			$sqlParts['where'][] = 'EXISTS ('.
 					'SELECT NULL'.
@@ -164,7 +160,7 @@ class CItem extends CItemGeneral {
 
 			$sqlParts['where']['hostid'] = dbConditionInt('i.hostid', $options['hostids']);
 
-			if (!is_null($options['groupCount'])) {
+			if ($options['groupCount']) {
 				$sqlParts['group']['i'] = 'i.hostid';
 			}
 		}
@@ -173,9 +169,9 @@ class CItem extends CItemGeneral {
 		if (!is_null($options['interfaceids'])) {
 			zbx_value2array($options['interfaceids']);
 
-			$sqlParts['where']['interfaceid'] = dbConditionInt('i.interfaceid', $options['interfaceids']);
+			$sqlParts['where']['interfaceid'] = dbConditionId('i.interfaceid', $options['interfaceids']);
 
-			if (!is_null($options['groupCount'])) {
+			if ($options['groupCount']) {
 				$sqlParts['group']['i'] = 'i.interfaceid';
 			}
 		}
@@ -188,7 +184,7 @@ class CItem extends CItemGeneral {
 			$sqlParts['where'][] = dbConditionInt('hg.groupid', $options['groupids']);
 			$sqlParts['where'][] = 'hg.hostid=i.hostid';
 
-			if (!is_null($options['groupCount'])) {
+			if ($options['groupCount']) {
 				$sqlParts['group']['hg'] = 'hg.groupid';
 			}
 		}
@@ -198,10 +194,10 @@ class CItem extends CItemGeneral {
 			zbx_value2array($options['proxyids']);
 
 			$sqlParts['from']['hosts'] = 'hosts h';
-			$sqlParts['where'][] = dbConditionInt('h.proxy_hostid', $options['proxyids']);
+			$sqlParts['where'][] = dbConditionId('h.proxy_hostid', $options['proxyids']);
 			$sqlParts['where'][] = 'h.hostid=i.hostid';
 
-			if (!is_null($options['groupCount'])) {
+			if ($options['groupCount']) {
 				$sqlParts['group']['h'] = 'h.proxy_hostid';
 			}
 		}
@@ -282,6 +278,19 @@ class CItem extends CItemGeneral {
 
 		// filter
 		if (is_array($options['filter'])) {
+			if (array_key_exists('delay', $options['filter']) && $options['filter']['delay'] !== null) {
+				$sqlParts['where'][] = makeUpdateIntervalFilter('i.delay', $options['filter']['delay']);
+				unset($options['filter']['delay']);
+			}
+
+			if (array_key_exists('history', $options['filter']) && $options['filter']['history'] !== null) {
+				$options['filter']['history'] = getTimeUnitFilters($options['filter']['history']);
+			}
+
+			if (array_key_exists('trends', $options['filter']) && $options['filter']['trends'] !== null) {
+				$options['filter']['trends'] = getTimeUnitFilters($options['filter']['trends']);
+			}
+
 			$this->dbFilter('items i', $options, $sqlParts);
 
 			if (isset($options['filter']['host'])) {
@@ -300,7 +309,7 @@ class CItem extends CItemGeneral {
 
 		// group
 		if (!is_null($options['group'])) {
-			$sqlParts['from']['groups'] = 'groups g';
+			$sqlParts['from']['hstgrp'] = 'hstgrp g';
 			$sqlParts['from']['hosts_groups'] = 'hosts_groups hg';
 			$sqlParts['where']['ghg'] = 'g.groupid=hg.groupid';
 			$sqlParts['where']['hgi'] = 'hg.hostid=i.hostid';
@@ -354,8 +363,8 @@ class CItem extends CItemGeneral {
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($item = DBfetch($res)) {
-			if (!is_null($options['countOutput'])) {
-				if (!is_null($options['groupCount'])) {
+			if ($options['countOutput']) {
+				if ($options['groupCount']) {
 					$result[] = $item;
 				}
 				else {
@@ -367,114 +376,38 @@ class CItem extends CItemGeneral {
 			}
 		}
 
-		if (!is_null($options['countOutput'])) {
+		if ($options['countOutput']) {
 			return $result;
 		}
 
 		// add other related objects
 		if ($result) {
 			$result = $this->addRelatedObjects($options, $result);
-			$result = $this->unsetExtraFields($result, array('hostid', 'interfaceid', 'value_type'), $options['output']);
+			$result = $this->unsetExtraFields($result, ['hostid', 'interfaceid', 'value_type'], $options['output']);
 		}
 
 		// removing keys (hash -> array)
-		if (is_null($options['preservekeys'])) {
+		if (!$options['preservekeys']) {
 			$result = zbx_cleanHashes($result);
 		}
 
+		// Decode ITEM_TYPE_HTTPAGENT encoded fields.
+		$json = new CJson();
+
+		foreach ($result as &$item) {
+			if (array_key_exists('query_fields', $item)) {
+				$query_fields = ($item['query_fields'] !== '') ? $json->decode($item['query_fields'], true) : [];
+				$item['query_fields'] = $json->hasError() ? [] : $query_fields;
+			}
+
+			if (array_key_exists('headers', $item)) {
+				$item['headers'] = $this->headersStringToArray($item['headers']);
+			}
+		}
+
 		return $result;
 	}
 
-	/**
-	 * Get items by item key and hostid.
-	 *
-	 * @deprecated	As of version 2.4, use get method instead.
-	 *
-	 * @param array  $itemData
-	 * @param string $itemData['key_']
-	 * @param string $itemData['hostid']
-	 *
-	 * @return array
-	 */
-	public function getObjects(array $itemData) {
-		$this->deprecated('item.getobjects method is deprecated.');
-
-		return $this->get(array(
-			'output' => API_OUTPUT_EXTEND,
-			'webitems' => true,
-			'filter' => $itemData
-		));
-	}
-
-	/**
-	 * Check if item exists.
-	 *
-	 * @deprecated	As of version 2.4, use get method instead.
-	 *
-	 * @param array $object
-	 *
-	 * @return bool
-	 */
-	public function exists(array $object) {
-		$this->deprecated('item.exists method is deprecated.');
-
-		$options = array(
-			'output' => array('itemid'),
-			'filter' => array('key_' => $object['key_']),
-			'webitems' => true,
-			'limit' => 1
-		);
-
-		if (isset($object['hostid'])) {
-			$options['hostids'] = $object['hostid'];
-		}
-		if (isset($object['host'])) {
-			$options['filter']['host'] = $object['host'];
-		}
-
-		$item = $this->get($options);
-
-		return (bool) $item;
-	}
-
-	/**
-	 * Check item data and set flags field.
-	 *
-	 * @param array $items
-	 * @param bool  $update
-	 *
-	 * @return void
-	 */
-	protected function checkInput(array &$items, $update = false) {
-		parent::checkInput($items, $update);
-		self::validateInventoryLinks($items, $update);
-
-		// set proper flags to divide normal and discovered items in future processing
-		if ($update) {
-			$dbItems = $this->get(array(
-				'itemids' => zbx_objectValues($items, 'itemid'),
-				'output' => array('itemid', 'flags'),
-				'editable' => true,
-				'preservekeys' => true
-			));
-			foreach ($items as &$item) {
-				$item['flags'] = $dbItems[$item['itemid']]['flags'];
-			}
-			unset($item);
-		}
-		else {
-			foreach ($items as &$item) {
-				$item['flags'] = ZBX_FLAG_DISCOVERY_NORMAL;
-
-				// set default formula value
-				if (!isset($item['formula'])) {
-					$item['formula'] = '1';
-				}
-			}
-			unset($item);
-		}
-	}
-
 	/**
 	 * Create item.
 	 *
@@ -485,11 +418,45 @@ class CItem extends CItemGeneral {
 	public function create($items) {
 		$items = zbx_toArray($items);
 
-		$this->checkInput($items);
+		parent::checkInput($items);
+		self::validateInventoryLinks($items);
+
+		foreach ($items as &$item) {
+			$item['flags'] = ZBX_FLAG_DISCOVERY_NORMAL;
+			unset($item['itemid']);
+		}
+		unset($item);
+
+		$this->validateDependentItems($items);
+
+		$json = new CJson();
+
+		foreach ($items as &$item) {
+			if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
+				if (array_key_exists('query_fields', $item)) {
+					$item['query_fields'] = $item['query_fields'] ? $json->encode($item['query_fields']) : '';
+				}
+
+				if (array_key_exists('headers', $item)) {
+					$item['headers'] = $this->headersArrayToString($item['headers']);
+				}
+
+				if (array_key_exists('request_method', $item) && $item['request_method'] == HTTPCHECK_REQUEST_HEAD
+						&& !array_key_exists('retrieve_mode', $item)) {
+					$item['retrieve_mode'] = HTTPTEST_STEP_RETRIEVE_MODE_HEADERS;
+				}
+			}
+			else {
+				$item['query_fields'] = '';
+				$item['headers'] = '';
+			}
+		}
+		unset($item);
+
 		$this->createReal($items);
 		$this->inherit($items);
 
-		return array('itemids' => zbx_objectValues($items, 'itemid'));
+		return ['itemids' => zbx_objectValues($items, 'itemid')];
 	}
 
 	/**
@@ -498,9 +465,16 @@ class CItem extends CItemGeneral {
 	 * @param array $items
 	 */
 	protected function createReal(array &$items) {
+		foreach ($items as &$item) {
+			if ($item['type'] != ITEM_TYPE_DEPENDENT) {
+				$item['master_itemid'] = null;
+			}
+		}
+		unset($item);
+
 		$itemids = DB::insert('items', $items);
 
-		$itemApplications = array();
+		$itemApplications = [];
 		foreach ($items as $key => $item) {
 			$items[$key]['itemid'] = $itemids[$key];
 
@@ -513,50 +487,37 @@ class CItem extends CItemGeneral {
 					continue;
 				}
 
-				$itemApplications[] = array(
+				$itemApplications[] = [
 					'applicationid' => $appid,
 					'itemid' => $items[$key]['itemid']
-				);
+				];
 			}
 		}
 
-		if (!empty($itemApplications)) {
-			DB::insert('items_applications', $itemApplications);
+		if ($itemApplications) {
+			DB::insertBatch('items_applications', $itemApplications);
 		}
 
-		$itemHosts = $this->get(array(
-			'itemids' => $itemids,
-			'output' => array('name'),
-			'selectHosts' => array('name'),
-			'nopermissions' => true
-		));
-		foreach ($itemHosts as $item) {
-			$host = reset($item['hosts']);
-			info(_s('Created: Item "%1$s" on "%2$s".', $item['name'], $host['name']));
-		}
+		$this->createItemPreprocessing($items);
 	}
 
 	/**
 	 * Update host items.
 	 *
 	 * @param array $items
-	 *
-	 * @return void
 	 */
 	protected function updateReal(array $items) {
-		$items = zbx_toArray($items);
+		CArrayHelper::sort($items, ['itemid']);
 
-		$itemids = array();
-		$data = array();
+		$data = [];
 		foreach ($items as $item) {
 			unset($item['flags']); // flags cannot be changed
-			$data[] = array('values' => $item, 'where' => array('itemid' => $item['itemid']));
-			$itemids[] = $item['itemid'];
+			$data[] = ['values' => $item, 'where' => ['itemid' => $item['itemid']]];
 		}
 		DB::update('items', $data);
 
-		$itemApplications = array();
-		$applicationids = array();
+		$itemApplications = [];
+		$applicationids = [];
 		foreach ($items as $item) {
 			if (!isset($item['applications'])) {
 				continue;
@@ -564,28 +525,19 @@ class CItem extends CItemGeneral {
 			$applicationids[] = $item['itemid'];
 
 			foreach ($item['applications'] as $appid) {
-				$itemApplications[] = array(
+				$itemApplications[] = [
 					'applicationid' => $appid,
 					'itemid' => $item['itemid']
-				);
+				];
 			}
 		}
 
 		if (!empty($applicationids)) {
-			DB::delete('items_applications', array('itemid' => $applicationids));
-			DB::insert('items_applications', $itemApplications);
+			DB::delete('items_applications', ['itemid' => $applicationids]);
+			DB::insertBatch('items_applications', $itemApplications);
 		}
 
-		$itemHosts = $this->get(array(
-			'itemids' => $itemids,
-			'output' => array('name'),
-			'selectHosts' => array('name'),
-			'nopermissions' => true
-		));
-		foreach ($itemHosts as $item) {
-			$host = reset($item['hosts']);
-			info(_s('Updated: Item "%1$s" on "%2$s".', $item['name'], $host['name']));
-		}
+		$this->updateItemPreprocessing($items);
 	}
 
 	/**
@@ -598,209 +550,236 @@ class CItem extends CItemGeneral {
 	public function update($items) {
 		$items = zbx_toArray($items);
 
-		$this->checkInput($items, true);
+		parent::checkInput($items, true);
+		self::validateInventoryLinks($items, true);
+
+		$db_items = $this->get([
+			'output' => ['flags', 'type', 'master_itemid', 'authtype', 'allow_traps', 'retrieve_mode'],
+			'itemids' => zbx_objectValues($items, 'itemid'),
+			'editable' => true,
+			'preservekeys' => true
+		]);
+
+		$items = $this->extendFromObjects(zbx_toHash($items, 'itemid'), $db_items, ['flags', 'type', 'master_itemid']);
+
+		$this->validateDependentItems($items);
+
+		$defaults = DB::getDefaults('items');
+		$clean = [
+			ITEM_TYPE_HTTPAGENT => [
+				'url' => '',
+				'query_fields' => '',
+				'timeout' => $defaults['timeout'],
+				'status_codes' => $defaults['status_codes'],
+				'follow_redirects' => $defaults['follow_redirects'],
+				'request_method' => $defaults['request_method'],
+				'allow_traps' => $defaults['allow_traps'],
+				'post_type' => $defaults['post_type'],
+				'http_proxy' => '',
+				'headers' => '',
+				'retrieve_mode' => $defaults['retrieve_mode'],
+				'output_format' => $defaults['output_format'],
+				'ssl_key_password' => '',
+				'verify_peer' => $defaults['verify_peer'],
+				'verify_host' => $defaults['verify_host'],
+				'ssl_cert_file' => '',
+				'ssl_key_file' => '',
+				'posts' => ''
+			]
+		];
+
+		$json = new CJson();
+
+		foreach ($items as &$item) {
+			$type_change = ($item['type'] != $db_items[$item['itemid']]['type']);
+
+			if ($item['type'] != ITEM_TYPE_DEPENDENT && $db_items[$item['itemid']]['master_itemid'] != 0) {
+				$item['master_itemid'] = 0;
+			}
+
+			if ($type_change && $db_items[$item['itemid']]['type'] == ITEM_TYPE_HTTPAGENT) {
+				$item = array_merge($item, $clean[ITEM_TYPE_HTTPAGENT]);
+
+				if ($item['type'] != ITEM_TYPE_SSH) {
+					$item['authtype'] = $defaults['authtype'];
+					$item['username'] = '';
+					$item['password'] = '';
+				}
+
+				if ($item['type'] != ITEM_TYPE_TRAPPER) {
+					$item['trapper_hosts'] = '';
+				}
+			}
+
+			if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
+				// Clean username and password on authtype change to HTTPTEST_AUTH_NONE.
+				if (array_key_exists('authtype', $item) && $item['authtype'] == HTTPTEST_AUTH_NONE
+						&& $item['authtype'] != $db_items[$item['itemid']]['authtype']) {
+					$item['username'] = '';
+					$item['password'] = '';
+				}
+
+				if (array_key_exists('allow_traps', $item) && $item['allow_traps'] == HTTPCHECK_ALLOW_TRAPS_OFF
+						&& $item['allow_traps'] != $db_items[$item['itemid']]['allow_traps']) {
+					$item['trapper_hosts'] = '';
+				}
+
+				if (array_key_exists('query_fields', $item) && is_array($item['query_fields'])) {
+					$item['query_fields'] = $item['query_fields'] ? $json->encode($item['query_fields']) : '';
+				}
+
+				if (array_key_exists('headers', $item) && is_array($item['headers'])) {
+					$item['headers'] = $this->headersArrayToString($item['headers']);
+				}
+
+				if (array_key_exists('request_method', $item) && $item['request_method'] == HTTPCHECK_REQUEST_HEAD
+						&& !array_key_exists('retrieve_mode', $item)
+						&& $db_items[$item['itemid']]['retrieve_mode'] != HTTPTEST_STEP_RETRIEVE_MODE_HEADERS) {
+					$item['retrieve_mode'] = HTTPTEST_STEP_RETRIEVE_MODE_HEADERS;
+				}
+			}
+			else {
+				$item['query_fields'] = '';
+				$item['headers'] = '';
+			}
+		}
+		unset($item);
+
 		$this->updateReal($items);
 		$this->inherit($items);
 
-		return array('itemids' => zbx_objectValues($items, 'itemid'));
+		return ['itemids' => zbx_objectValues($items, 'itemid')];
 	}
 
 	/**
 	 * Delete items.
 	 *
-	 * @param array $itemIds
-	 * @param bool  $noPermissions
+	 * @param array $itemids
 	 *
 	 * @return array
 	 */
-	public function delete(array $itemIds, $noPermissions = false) {
-		if (!$itemIds) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Empty input parameter.'));
+	public function delete(array $itemids) {
+		$this->validateDelete($itemids, $db_items);
+
+		CItemManager::delete($itemids);
+
+		$this->addAuditBulk(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_ITEM, $db_items);
+
+		return ['itemids' => $itemids];
+	}
+
+	/**
+	 * Validates the input parameters for the delete() method.
+	 *
+	 * @param array $itemids   [IN/OUT]
+	 * @param array $db_items  [OUT]
+	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	private function validateDelete(array &$itemids, array &$db_items = null) {
+		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
+		if (!CApiInputValidator::validate($api_input_rules, $itemids, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
-		$itemIds = array_keys(array_flip($itemIds));
-
-		$delItems = $this->get(array(
-			'output' => array('name', 'templateid'),
-			'selectHosts' => array('name'),
-			'itemids' => $itemIds,
+		$db_items = $this->get([
+			'output' => ['itemid', 'name', 'templateid', 'flags'],
+			'itemids' => $itemids,
 			'editable' => true,
 			'preservekeys' => true
-		));
+		]);
 
-		// TODO: remove $nopermissions hack
-		if (!$noPermissions) {
-			foreach ($itemIds as $itemId) {
-				if (!isset($delItems[$itemId])) {
-					self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
-				}
-				if ($delItems[$itemId]['templateid'] != 0) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot delete templated item.'));
-				}
-			}
-		}
-
-		// first delete child items
-		$parentItemIds = $itemIds;
-		do {
-			$dbItems = DBselect('SELECT i.itemid FROM items i WHERE '.dbConditionInt('i.templateid', $parentItemIds));
-			$parentItemIds = array();
-			while ($dbItem = DBfetch($dbItems)) {
-				$parentItemIds[] = $dbItem['itemid'];
-				$itemIds[$dbItem['itemid']] = $dbItem['itemid'];
-			}
-		} while ($parentItemIds);
-
-		// delete graphs, leave if graph still have item
-		$delGraphs = array();
-		$dbGraphs = DBselect(
-			'SELECT gi.graphid'.
-			' FROM graphs_items gi'.
-			' WHERE '.dbConditionInt('gi.itemid', $itemIds).
-				' AND NOT EXISTS ('.
-					'SELECT NULL'.
-					' FROM graphs_items gii'.
-					' WHERE gii.graphid=gi.graphid'.
-						' AND '.dbConditionInt('gii.itemid', $itemIds, true).
-				')'
-		);
-		while ($dbGraph = DBfetch($dbGraphs)) {
-			$delGraphs[$dbGraph['graphid']] = $dbGraph['graphid'];
-		}
-
-		if ($delGraphs) {
-			API::Graph()->delete($delGraphs, true);
-		}
-
-		// check if any graphs are referencing this item
-		$this->checkGraphReference($itemIds);
-
-		$triggers = API::Trigger()->get(array(
-			'output' => array(),
-			'itemids' => $itemIds,
-			'nopermissions' => true,
-			'preservekeys' => true
-		));
-		if ($triggers) {
-			API::Trigger()->delete(array_keys($triggers), true);
-		}
-
-		$triggerPrototypes = API::TriggerPrototype()->get(array(
-			'output' => array(),
-			'itemids' => $itemIds,
-			'nopermissions' => true,
-			'preservekeys' => true
-		));
-		if ($triggerPrototypes) {
-			API::TriggerPrototype()->delete(array_keys($triggerPrototypes), true);
-		}
-
-		DB::delete('screens_items', array(
-			'resourceid' => $itemIds,
-			'resourcetype' => array(SCREEN_RESOURCE_SIMPLE_GRAPH, SCREEN_RESOURCE_PLAIN_TEXT, SCREEN_RESOURCE_CLOCK)
-		));
-
-		DB::delete('items', array('itemid' => $itemIds));
-		DB::delete('profiles', array(
-			'idx' => 'web.favorite.graphids',
-			'source' => 'itemid',
-			'value_id' => $itemIds
-		));
-
-		$itemDataTables = array(
-			'trends',
-			'trends_uint',
-			'history_text',
-			'history_log',
-			'history_uint',
-			'history_str',
-			'history'
-		);
-		$insert = array();
-		foreach ($itemIds as $itemId) {
-			foreach ($itemDataTables as $table) {
-				$insert[] = array(
-					'tablename' => $table,
-					'field' => 'itemid',
-					'value' => $itemId
+		foreach ($itemids as $itemid) {
+			if (!array_key_exists($itemid, $db_items)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
 				);
 			}
-		}
-		DB::insert('housekeeper', $insert);
 
-		// TODO: remove info from API
-		foreach ($delItems as $item) {
-			$host = reset($item['hosts']);
-			info(_s('Deleted: Item "%1$s" on "%2$s".', $item['name'], $host['name']));
-		}
+			$db_item = $db_items[$itemid];
 
-		return array('itemids' => $itemIds);
+			if ($db_item['templateid'] != 0) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot delete templated item.'));
+			}
+		}
 	}
 
 	public function syncTemplates($data) {
 		$data['templateids'] = zbx_toArray($data['templateids']);
 		$data['hostids'] = zbx_toArray($data['hostids']);
 
-		$selectFields = array();
-		foreach ($this->fieldRules as $key => $rules) {
-			if (!isset($rules['system']) && !isset($rules['host'])) {
-				$selectFields[] = $key;
+		$output = [];
+		foreach ($this->fieldRules as $field_name => $rules) {
+			if (!array_key_exists('system', $rules) && !array_key_exists('host', $rules)) {
+				$output[] = $field_name;
 			}
 		}
 
-		$items = $this->get(array(
+		$tpl_items = $this->get([
+			'output' => $output,
+			'selectApplications' => ['applicationid'],
+			'selectPreprocessing' => ['type', 'params'],
 			'hostids' => $data['templateids'],
-			'preservekeys' => true,
-			'selectApplications' => array('applicationid'),
-			'output' => $selectFields,
-			'filter' => array('flags' => ZBX_FLAG_DISCOVERY_NORMAL)
-		));
+			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL],
+			'preservekeys' => true
+		]);
 
-		foreach ($items as $inum => $item) {
-			$items[$inum]['applications'] = zbx_objectValues($item['applications'], 'applicationid');
+		$json = new CJson();
+
+		foreach ($tpl_items as &$tpl_item) {
+			$tpl_item['applications'] = zbx_objectValues($tpl_item['applications'], 'applicationid');
+
+			if ($tpl_item['type'] == ITEM_TYPE_HTTPAGENT) {
+				if (array_key_exists('query_fields', $tpl_item) && is_array($tpl_item['query_fields'])) {
+					$tpl_item['query_fields'] = $tpl_item['query_fields']
+						? $json->encode($tpl_item['query_fields'])
+						: '';
+				}
+
+				if (array_key_exists('headers', $tpl_item) && is_array($tpl_item['headers'])) {
+					$tpl_item['headers'] = $this->headersArrayToString($tpl_item['headers']);
+				}
+			}
+			else {
+				$tpl_item['query_fields'] = '';
+				$tpl_item['headers'] = '';
+			}
 		}
+		unset($tpl_item);
 
-		$this->inherit($items, $data['hostids']);
+		$this->inherit($tpl_items, $data['hostids']);
 
 		return true;
 	}
 
-	protected function inherit(array $items, array $hostids = null) {
-		if (empty($items)) {
-			return true;
+	/**
+	 * Check item specific fields:
+	 *		- validate history and trends using simple interval parser and user macro parser;
+	 *		- validate item preprocessing.
+	 *
+	 * @param array  $item    An array of single item data.
+	 * @param string $method  A string of "create" or "update" method.
+	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	protected function checkSpecificFields(array $item, $method) {
+		if (array_key_exists('history', $item)
+				&& !validateTimeUnit($item['history'], SEC_PER_HOUR, 25 * SEC_PER_YEAR, true, $error,
+					['usermacros' => true])) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Incorrect value for field "%1$s": %2$s.', 'history', $error)
+			);
 		}
 
-		// prepare the child items
-		$newItems = $this->prepareInheritedItems($items, $hostids);
-		if (!$newItems) {
-			return true;
+		if (array_key_exists('trends', $item)
+				&& !validateTimeUnit($item['trends'], SEC_PER_DAY, 25 * SEC_PER_YEAR, true, $error,
+					['usermacros' => true])) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Incorrect value for field "%1$s": %2$s.', 'trends', $error)
+			);
 		}
 
-		$insertItems = array();
-		$updateItems = array();
-		foreach ($newItems as $newItem) {
-			if (isset($newItem['itemid'])) {
-				$updateItems[] = $newItem;
-			}
-			else {
-				$newItem['flags'] = ZBX_FLAG_DISCOVERY_NORMAL;
-				$insertItems[] = $newItem;
-			}
-		}
-
-		// save the new items
-		if (!zbx_empty($insertItems)) {
-			self::validateInventoryLinks($insertItems, false); // false means 'create'
-			$this->createReal($insertItems);
-		}
-
-		if (!zbx_empty($updateItems)) {
-			self::validateInventoryLinks($updateItems, true); // true means 'update'
-			$this->updateReal($updateItems);
-		}
-
-		// propagate the inheritance to the children
-		return $this->inherit(array_merge($updateItems, $insertItems));
+		$this->validateItemPreprocessing($item, $method);
 	}
 
 	/**
@@ -832,9 +811,9 @@ class CItem extends CItemGeneral {
 			// for successful validation we need three fields for each item: inventory_link, hostid and key_
 			// problem is, that when we are updating an item, we might not have them, because they are not changed
 			// so, we need to find out what is missing and use API to get the lacking info
-			$itemsWithNoHostId = array();
-			$itemsWithNoInventoryLink = array();
-			$itemsWithNoKeys = array();
+			$itemsWithNoHostId = [];
+			$itemsWithNoInventoryLink = [];
+			$itemsWithNoKeys = [];
 			foreach ($items as $item) {
 				if (!isset($item['inventory_link'])) {
 					$itemsWithNoInventoryLink[$item['itemid']] = $item['itemid'];
@@ -850,11 +829,11 @@ class CItem extends CItemGeneral {
 
 			// are there any items with lacking info?
 			if (!zbx_empty($itemsToFind)) {
-				$missingInfo = API::Item()->get(array(
-					'output' => array('hostid', 'inventory_link', 'key_'),
-					'filter' => array('itemid' => $itemsToFind),
+				$missingInfo = API::Item()->get([
+					'output' => ['hostid', 'inventory_link', 'key_'],
+					'filter' => ['itemid' => $itemsToFind],
 					'nopermissions' => true
-				));
+				]);
 				$missingInfo = zbx_toHash($missingInfo, 'itemid');
 
 				// appending host ids, inventory_links and keys where they are needed
@@ -877,19 +856,19 @@ class CItem extends CItemGeneral {
 		$hostids = zbx_objectValues($items, 'hostid');
 
 		// getting all inventory links on every affected host
-		$itemsOnHostsInfo = API::Item()->get(array(
-			'output' => array('key_', 'inventory_link', 'hostid'),
-			'filter' => array('hostid' => $hostids),
+		$itemsOnHostsInfo = API::Item()->get([
+			'output' => ['key_', 'inventory_link', 'hostid'],
+			'filter' => ['hostid' => $hostids],
 			'nopermissions' => true
-		));
+		]);
 
 		// now, changing array to: 'hostid' => array('key_'=>'inventory_link')
-		$linksOnHostsCurr = array();
+		$linksOnHostsCurr = [];
 		foreach ($itemsOnHostsInfo as $info) {
 			// 0 means no link - we are not interested in those ones
 			if ($info['inventory_link'] != 0) {
 				if (!isset($linksOnHostsCurr[$info['hostid']])) {
-					$linksOnHostsCurr[$info['hostid']] = array($info['key_'] => $info['inventory_link']);
+					$linksOnHostsCurr[$info['hostid']] = [$info['key_'] => $info['inventory_link']];
 				}
 				else{
 					$linksOnHostsCurr[$info['hostid']][$info['key_']] = $info['inventory_link'];
@@ -897,7 +876,7 @@ class CItem extends CItemGeneral {
 			}
 		}
 
-		$linksOnHostsFuture = array();
+		$linksOnHostsFuture = [];
 
 		foreach ($items as $item) {
 			// checking if inventory_link value is a valid number
@@ -913,7 +892,7 @@ class CItem extends CItemGeneral {
 			}
 
 			if (!isset($linksOnHostsFuture[$item['hostid']])) {
-				$linksOnHostsFuture[$item['hostid']] = array($item['key_'] => $item['inventory_link']);
+				$linksOnHostsFuture[$item['hostid']] = [$item['key_'] => $item['inventory_link']];
 			}
 			else {
 				$linksOnHostsFuture[$item['hostid']][$item['key_']] = $item['inventory_link'];
@@ -943,11 +922,11 @@ class CItem extends CItemGeneral {
 							$beingSavedItemName = $item['name'];
 						}
 						else {
-							$thisItem = API::Item()->get(array(
-								'output' => array('name'),
-								'filter' => array('itemid' => $item['itemid']),
+							$thisItem = API::Item()->get([
+								'output' => ['name'],
+								'filter' => ['itemid' => $item['itemid']],
 								'nopermissions' => true
-							));
+							]);
 							$beingSavedItemName = $thisItem[0]['name'];
 						}
 						break;
@@ -955,14 +934,14 @@ class CItem extends CItemGeneral {
 				}
 
 				// name of the original item that already populates the field
-				$originalItem = API::Item()->get(array(
-					'output' => array('name'),
-					'filter' => array(
+				$originalItem = API::Item()->get([
+					'output' => ['name'],
+					'filter' => [
 						'hostid' => $hostId,
 						'inventory_link' => $conflictedLink
-					),
+					],
 					'nopermissions' => true
-				));
+				]);
 				$originalItemName = $originalItem[0]['name'];
 
 				self::exception(
@@ -988,23 +967,23 @@ class CItem extends CItemGeneral {
 		// adding applications
 		if ($options['selectApplications'] !== null && $options['selectApplications'] != API_OUTPUT_COUNT) {
 			$relationMap = $this->createRelationMap($result, 'itemid', 'applicationid', 'items_applications');
-			$applications = API::Application()->get(array(
+			$applications = API::Application()->get([
 				'output' => $options['selectApplications'],
 				'applicationids' => $relationMap->getRelatedIds(),
 				'preservekeys' => true
-			));
+			]);
 			$result = $relationMap->mapMany($result, $applications, 'applications');
 		}
 
 		// adding interfaces
 		if ($options['selectInterfaces'] !== null && $options['selectInterfaces'] != API_OUTPUT_COUNT) {
 			$relationMap = $this->createRelationMap($result, 'itemid', 'interfaceid');
-			$interfaces = API::HostInterface()->get(array(
+			$interfaces = API::HostInterface()->get([
 				'output' => $options['selectInterfaces'],
 				'interfaceids' => $relationMap->getRelatedIds(),
 				'nopermissions' => true,
 				'preservekeys' => true
-			));
+			]);
 			$result = $relationMap->mapMany($result, $interfaces, 'interfaces');
 		}
 
@@ -1012,11 +991,11 @@ class CItem extends CItemGeneral {
 		if (!is_null($options['selectTriggers'])) {
 			if ($options['selectTriggers'] != API_OUTPUT_COUNT) {
 				$relationMap = $this->createRelationMap($result, 'itemid', 'triggerid', 'functions');
-				$triggers = API::Trigger()->get(array(
+				$triggers = API::Trigger()->get([
 					'output' => $options['selectTriggers'],
 					'triggerids' => $relationMap->getRelatedIds(),
 					'preservekeys' => true
-				));
+				]);
 
 				if (!is_null($options['limitSelects'])) {
 					order_result($triggers, 'description');
@@ -1024,20 +1003,17 @@ class CItem extends CItemGeneral {
 				$result = $relationMap->mapMany($result, $triggers, 'triggers', $options['limitSelects']);
 			}
 			else {
-				$triggers = API::Trigger()->get(array(
+				$triggers = API::Trigger()->get([
 					'countOutput' => true,
 					'groupCount' => true,
 					'itemids' => $itemids
-				));
+				]);
 				$triggers = zbx_toHash($triggers, 'itemid');
 
 				foreach ($result as $itemid => $item) {
-					if (isset($triggers[$itemid])) {
-						$result[$itemid]['triggers'] = $triggers[$itemid]['rowscount'];
-					}
-					else {
-						$result[$itemid]['triggers'] = 0;
-					}
+					$result[$itemid]['triggers'] = array_key_exists($itemid, $triggers)
+						? $triggers[$itemid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1046,11 +1022,11 @@ class CItem extends CItemGeneral {
 		if (!is_null($options['selectGraphs'])) {
 			if ($options['selectGraphs'] != API_OUTPUT_COUNT) {
 				$relationMap = $this->createRelationMap($result, 'itemid', 'graphid', 'graphs_items');
-				$graphs = API::Graph()->get(array(
+				$graphs = API::Graph()->get([
 					'output' => $options['selectGraphs'],
 					'graphids' => $relationMap->getRelatedIds(),
 					'preservekeys' => true
-				));
+				]);
 
 				if (!is_null($options['limitSelects'])) {
 					order_result($graphs, 'name');
@@ -1058,20 +1034,17 @@ class CItem extends CItemGeneral {
 				$result = $relationMap->mapMany($result, $graphs, 'graphs', $options['limitSelects']);
 			}
 			else {
-				$graphs = API::Graph()->get(array(
+				$graphs = API::Graph()->get([
 					'countOutput' => true,
 					'groupCount' => true,
 					'itemids' => $itemids
-				));
+				]);
 				$graphs = zbx_toHash($graphs, 'itemid');
 
 				foreach ($result as $itemid => $item) {
-					if (isset($graphs[$itemid])) {
-						$result[$itemid]['graphs'] = $graphs[$itemid]['rowscount'];
-					}
-					else {
-						$result[$itemid]['graphs'] = 0;
-					}
+					$result[$itemid]['graphs'] = array_key_exists($itemid, $graphs)
+						? $graphs[$itemid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1105,32 +1078,32 @@ class CItem extends CItemGeneral {
 				$relationMap->addRelation($rule['itemid'], $rule['parent_itemid']);
 			}
 
-			$discoveryRules = API::DiscoveryRule()->get(array(
+			$discoveryRules = API::DiscoveryRule()->get([
 				'output' => $options['selectDiscoveryRule'],
 				'itemids' => $relationMap->getRelatedIds(),
 				'nopermissions' => true,
 				'preservekeys' => true
-			));
+			]);
 			$result = $relationMap->mapOne($result, $discoveryRules, 'discoveryRule');
 		}
 
 		// adding item discovery
 		if ($options['selectItemDiscovery'] !== null) {
-			$itemDiscoveries = API::getApiService()->select('item_discovery', array(
-				'output' => $this->outputExtend($options['selectItemDiscovery'], array('itemdiscoveryid', 'itemid')),
-				'filter' => array('itemid' => array_keys($result)),
+			$itemDiscoveries = API::getApiService()->select('item_discovery', [
+				'output' => $this->outputExtend($options['selectItemDiscovery'], ['itemdiscoveryid', 'itemid']),
+				'filter' => ['itemid' => array_keys($result)],
 				'preservekeys' => true
-			));
+			]);
 			$relationMap = $this->createRelationMap($itemDiscoveries, 'itemid', 'itemdiscoveryid');
 
-			$itemDiscoveries = $this->unsetExtraFields($itemDiscoveries, array('itemid', 'itemdiscoveryid'),
+			$itemDiscoveries = $this->unsetExtraFields($itemDiscoveries, ['itemid', 'itemdiscoveryid'],
 				$options['selectItemDiscovery']
 			);
 			$result = $relationMap->mapOne($result, $itemDiscoveries, 'itemDiscovery');
 		}
 
 		// adding history data
-		$requestedOutput = array();
+		$requestedOutput = [];
 		if ($this->outputIsRequested('lastclock', $options['output'])) {
 			$requestedOutput['lastclock'] = true;
 		}
@@ -1144,10 +1117,12 @@ class CItem extends CItemGeneral {
 			$requestedOutput['prevvalue'] = true;
 		}
 		if ($requestedOutput) {
-			$history = Manager::History()->getLast($result, 2, ZBX_HISTORY_PERIOD);
+			$history = Manager::History()->getLastValues($result, 2, ZBX_HISTORY_PERIOD);
 			foreach ($result as &$item) {
 				$lastHistory = isset($history[$item['itemid']][0]) ? $history[$item['itemid']][0] : null;
 				$prevHistory = isset($history[$item['itemid']][1]) ? $history[$item['itemid']][1] : null;
+				$no_value = in_array($item['value_type'],
+						[ITEM_VALUE_TYPE_STR, ITEM_VALUE_TYPE_LOG, ITEM_VALUE_TYPE_TEXT]) ? '' : '0';
 
 				if (isset($requestedOutput['lastclock'])) {
 					$item['lastclock'] = $lastHistory ? $lastHistory['clock'] : '0';
@@ -1156,22 +1131,45 @@ class CItem extends CItemGeneral {
 					$item['lastns'] = $lastHistory ? $lastHistory['ns'] : '0';
 				}
 				if (isset($requestedOutput['lastvalue'])) {
-					$item['lastvalue'] = $lastHistory ? $lastHistory['value'] : '0';
+					$item['lastvalue'] = $lastHistory ? $lastHistory['value'] : $no_value;
 				}
 				if (isset($requestedOutput['prevvalue'])) {
-					$item['prevvalue'] = $prevHistory ? $prevHistory['value'] : '0';
+					$item['prevvalue'] = $prevHistory ? $prevHistory['value'] : $no_value;
 				}
 			}
 			unset($item);
 		}
 
+		if ($options['selectPreprocessing'] !== null && $options['selectPreprocessing'] != API_OUTPUT_COUNT) {
+			$db_item_preproc = API::getApiService()->select('item_preproc', [
+				'output' => $this->outputExtend($options['selectPreprocessing'], ['itemid', 'step']),
+				'filter' => ['itemid' => array_keys($result)],
+			]);
+
+			CArrayHelper::sort($db_item_preproc, ['step']);
+
+			foreach ($result as &$item) {
+				$item['preprocessing'] = [];
+			}
+			unset($item);
+
+			foreach ($db_item_preproc as $step) {
+				$itemid = $step['itemid'];
+				unset($step['item_preprocid'], $step['itemid'], $step['step']);
+
+				if (array_key_exists($itemid, $result)) {
+					$result[$itemid]['preprocessing'][] = $step;
+				}
+			}
+		}
+
 		return $result;
 	}
 
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
 		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
 
-		if ($options['countOutput'] === null) {
+		if (!$options['countOutput']) {
 			if ($options['selectHosts'] !== null) {
 				$sqlParts = $this->addQuerySelect('i.hostid', $sqlParts);
 			}
