@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -47,8 +47,6 @@ class CTemplate extends CHostGeneral {
 	 */
 	public function get($options = []) {
 		$result = [];
-		$userType = self::$userData['type'];
-		$userid = self::$userData['userid'];
 
 		$sqlParts = [
 			'select'	=> ['templates' => 'h.hostid'],
@@ -71,14 +69,14 @@ class CTemplate extends CHostGeneral {
 			'with_triggers'				=> null,
 			'with_graphs'				=> null,
 			'with_httptests'			=> null,
-			'editable'					=> null,
+			'editable'					=> false,
 			'nopermissions'				=> null,
 			// filter
 			'filter'					=> null,
 			'search'					=> '',
 			'searchByAny'				=> null,
-			'startSearch'				=> null,
-			'excludeSearch'				=> null,
+			'startSearch'				=> false,
+			'excludeSearch'				=> false,
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
@@ -94,9 +92,9 @@ class CTemplate extends CHostGeneral {
 			'selectMacros'				=> null,
 			'selectScreens'				=> null,
 			'selectHttpTests'			=> null,
-			'countOutput'				=> null,
-			'groupCount'				=> null,
-			'preservekeys'				=> null,
+			'countOutput'				=> false,
+			'groupCount'				=> false,
+			'preservekeys'				=> false,
 			'sortfield'					=> '',
 			'sortorder'					=> '',
 			'limit'						=> null,
@@ -105,10 +103,9 @@ class CTemplate extends CHostGeneral {
 		$options = zbx_array_merge($defOptions, $options);
 
 		// editable + PERMISSION CHECK
-		if ($userType != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
 			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
-
-			$userGroups = getUserGroupsByUserId($userid);
+			$userGroups = getUserGroupsByUserId(self::$userData['userid']);
 
 			$sqlParts['where'][] = 'EXISTS ('.
 					'SELECT NULL'.
@@ -131,7 +128,7 @@ class CTemplate extends CHostGeneral {
 			$sqlParts['where'][] = dbConditionInt('hg.groupid', $options['groupids']);
 			$sqlParts['where']['hgh'] = 'hg.hostid=h.hostid';
 
-			if (!is_null($options['groupCount'])) {
+			if ($options['groupCount']) {
 				$sqlParts['group']['hg'] = 'hg.groupid';
 			}
 		}
@@ -151,7 +148,7 @@ class CTemplate extends CHostGeneral {
 			$sqlParts['where'][] = dbConditionInt('ht.templateid', $options['parentTemplateids']);
 			$sqlParts['where']['hht'] = 'h.hostid=ht.hostid';
 
-			if (!is_null($options['groupCount'])) {
+			if ($options['groupCount']) {
 				$sqlParts['group']['templateid'] = 'ht.templateid';
 			}
 		}
@@ -164,7 +161,7 @@ class CTemplate extends CHostGeneral {
 			$sqlParts['where'][] = dbConditionInt('ht.hostid', $options['hostids']);
 			$sqlParts['where']['hht'] = 'h.hostid=ht.templateid';
 
-			if (!is_null($options['groupCount'])) {
+			if ($options['groupCount']) {
 				$sqlParts['group']['ht'] = 'ht.hostid';
 			}
 		}
@@ -258,11 +255,13 @@ class CTemplate extends CHostGeneral {
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($template = DBfetch($res)) {
-			if (!is_null($options['countOutput'])) {
-				if (!is_null($options['groupCount']))
+			if ($options['countOutput']) {
+				if ($options['groupCount']) {
 					$result[] = $template;
-				else
+				}
+				else {
 					$result = $template['rowscount'];
+				}
 			}
 			else{
 				$template['templateid'] = $template['hostid'];
@@ -273,7 +272,7 @@ class CTemplate extends CHostGeneral {
 
 		}
 
-		if (!is_null($options['countOutput'])) {
+		if ($options['countOutput']) {
 			return $result;
 		}
 
@@ -282,7 +281,7 @@ class CTemplate extends CHostGeneral {
 		}
 
 		// removing keys (hash -> array)
-		if (is_null($options['preservekeys'])) {
+		if (!$options['preservekeys']) {
 			$result = zbx_cleanHashes($result);
 		}
 
@@ -365,7 +364,9 @@ class CTemplate extends CHostGeneral {
 		foreach ($templates as $template) {
 			// check if hosts have at least 1 group
 			if (!isset($template['groups']) || !$template['groups']) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('No groups for template "%1$s".', $template['host']));
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Template "%1$s" cannot be without host group.', $template['host'])
+				);
 			}
 
 			$template['groups'] = zbx_toArray($template['groups']);
@@ -390,6 +391,8 @@ class CTemplate extends CHostGeneral {
 
 		$templateDbFields = ['host' => null];
 
+		$host_name_parser = new CHostNameParser();
+
 		foreach ($templates as $template) {
 			// if visible name is not given or empty it should be set to host name
 			if ((!isset($template['name']) || zbx_empty(trim($template['name']))) && isset($template['host'])) {
@@ -400,11 +403,15 @@ class CTemplate extends CHostGeneral {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Field "host" is mandatory.'));
 			}
 
-			if (!preg_match('/^'.ZBX_PREG_HOST_FORMAT.'$/', $template['host'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s(
-					'Incorrect characters used for template name "%1$s".',
-					$template['host']
-				));
+			// Property 'auto_compress' is not supported for templates.
+			if (array_key_exists('auto_compress', $template)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect input parameters.'));
+			}
+
+			if ($host_name_parser->parse($template['host']) != CParser::PARSE_SUCCESS) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Incorrect characters used for template name "%1$s".', $template['host'])
+				);
 			}
 
 			if (isset($template['host'])) {
@@ -522,6 +529,11 @@ class CTemplate extends CHostGeneral {
 			if (!isset($dbTemplates[$template['templateid']])) {
 				self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
 			}
+
+			// Property 'auto_compress' is not supported for templates.
+			if (array_key_exists('auto_compress', $template)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect input parameters.'));
+			}
 		}
 	}
 
@@ -554,25 +566,25 @@ class CTemplate extends CHostGeneral {
 		API::Template()->unlink($templateids, null, true);
 
 		// delete the discovery rules first
-		$delRules = API::DiscoveryRule()->get([
-			'output' => ['itemid'],
+		$del_rules = API::DiscoveryRule()->get([
+			'output' => [],
 			'hostids' => $templateids,
 			'nopermissions' => true,
 			'preservekeys' => true
 		]);
-		if ($delRules) {
-			API::DiscoveryRule()->delete(array_keys($delRules), true);
+		if ($del_rules) {
+			API::DiscoveryRule()->delete(array_keys($del_rules), true);
 		}
 
 		// delete the items
-		$delItems = API::Item()->get([
+		$del_items = API::Item()->get([
+			'output' => [],
 			'templateids' => $templateids,
-			'output' => ['itemid'],
 			'nopermissions' => true,
 			'preservekeys' => true
 		]);
-		if ($delItems) {
-			API::Item()->delete(array_keys($delItems), true);
+		if ($del_items) {
+			CItemManager::delete(array_keys($del_items));
 		}
 
 		// delete host from maps
@@ -649,7 +661,7 @@ class CTemplate extends CHostGeneral {
 			'templateids' => $templateids,
 			'output' => ['httptestid'],
 			'nopermissions' => 1,
-			'preservekeys' => 1
+			'preservekeys' => true
 		]);
 		if (!empty($delHttpTests)) {
 			API::HttpTest()->delete(array_keys($delHttpTests), true);
@@ -660,7 +672,7 @@ class CTemplate extends CHostGeneral {
 			'templateids' => $templateids,
 			'output' => ['applicationid'],
 			'nopermissions' => 1,
-			'preservekeys' => 1
+			'preservekeys' => true
 		]);
 		if (!empty($delApplications)) {
 			API::Application()->delete(array_keys($delApplications), true);
@@ -677,6 +689,43 @@ class CTemplate extends CHostGeneral {
 		return ['templateids' => $templateids];
 	}
 
+	/**
+	 * Checks if the current user has access to the given hosts and templates. Assumes the "hostid" field is valid.
+	 *
+	 * @param array $hostids    an array of host or template IDs
+	 *
+	 * @throws APIException if the user doesn't have write permissions for the given hosts.
+	 *
+	 * @return void
+	 */
+	protected function checkHostPermissions(array $hostids) {
+		if ($hostids) {
+			$hostids = array_unique($hostids);
+
+			$count = API::Host()->get([
+				'countOutput' => true,
+				'hostids' => $hostids,
+				'editable' => true
+			]);
+
+			if ($count == count($hostids)) {
+				return;
+			}
+
+			$count += $this->get([
+				'countOutput' => true,
+				'templateids' => $hostids,
+				'editable' => true
+			]);
+
+			if ($count != count($hostids)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
+		}
+	}
+
 	/**
 	 * Additionally allows to link templates to hosts and other templates.
 	 *
@@ -693,20 +742,13 @@ class CTemplate extends CHostGeneral {
 		$templates = isset($data['templates']) ? zbx_toArray($data['templates']) : [];
 		$templateIds = zbx_objectValues($templates, 'templateid');
 
-		// check permissions
-		if (!$this->isWritable($templateIds)) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
-		}
+		$this->checkPermissions($templateIds, _('No permissions to referred object or it does not exist!'));
 
 		// link hosts to the given templates
 		if (isset($data['hosts']) && !empty($data['hosts'])) {
 			$hostIds = zbx_objectValues($data['hosts'], 'hostid');
 
-			if (!API::Host()->isWritable($hostIds)) {
-				self::exception(ZBX_API_ERROR_PERMISSIONS,
-					_('No permissions to referred object or it does not exist!')
-				);
-			}
+			$this->checkHostPermissions($hostIds);
 
 			// check if any of the hosts are discovered
 			$this->checkValidator($hostIds, new CHostNormalValidator([
@@ -737,6 +779,10 @@ class CTemplate extends CHostGeneral {
 	 * @return array
 	 */
 	public function massUpdate(array $data) {
+		if (!array_key_exists('templates', $data) || !is_array($data['templates'])) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Field "%1$s" is mandatory.', 'templates'));
+		}
+
 		$this->validateMassUpdate($data);
 
 		$templates = zbx_toArray($data['templates']);
@@ -921,7 +967,7 @@ class CTemplate extends CHostGeneral {
 		$templates = zbx_toArray($data['templates']);
 
 		$dbTemplates = $this->get([
-			'output' => ['templateid'],
+			'output' => ['templateid', 'host'],
 			'templateids' => zbx_objectValues($templates, 'templateid'),
 			'editable' => true,
 			'preservekeys' => true
@@ -934,9 +980,12 @@ class CTemplate extends CHostGeneral {
 			}
 		}
 
-		// check if templates have at least 1 group
-		if (isset($data['groups']) && !$data['groups']) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('No groups for template.'));
+		if (array_key_exists('groups', $data) && !$data['groups'] && $dbTemplates) {
+			$template = reset($dbTemplates);
+
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Template "%1$s" cannot be without host group.', $template['host'])
+			);
 		}
 
 		// check name
@@ -1009,11 +1058,12 @@ class CTemplate extends CHostGeneral {
 			}
 		}
 
-		if (isset($data['host']) && !preg_match('/^'.ZBX_PREG_HOST_FORMAT.'$/', $data['host'])) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _s(
-				'Incorrect characters used for template name "%1$s".',
-				$data['host']
-			));
+		$host_name_parser = new CHostNameParser();
+
+		if (array_key_exists('host', $data) && $host_name_parser->parse($data['host']) != CParser::PARSE_SUCCESS) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Incorrect characters used for template name "%1$s".', $data['host'])
+			);
 		}
 	}
 
@@ -1032,10 +1082,7 @@ class CTemplate extends CHostGeneral {
 	public function massRemove(array $data) {
 		$templateids = zbx_toArray($data['templateids']);
 
-		// check permissions
-		if (!$this->isWritable($templateids)) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
-		}
+		$this->checkPermissions($templateids, _('You do not have permission to perform this operation.'));
 
 		if (isset($data['hostids'])) {
 			// check if any of the hosts are discovered
@@ -1051,55 +1098,28 @@ class CTemplate extends CHostGeneral {
 		return parent::massRemove($data);
 	}
 
-	/**
-	 * Check if user has read permissions for templates.
-	 *
-	 * @param array $ids
-	 *
-	 * @return bool
-	 */
-	public function isReadable(array $ids) {
-		if (!is_array($ids)) {
-			return false;
-		}
-		if (empty($ids)) {
-			return true;
-		}
-
-		$ids = array_unique($ids);
-
-		$count = $this->get([
-			'templateids' => $ids,
-			'countOutput' => true
-		]);
-
-		return (count($ids) == $count);
-	}
-
 	/**
 	 * Check if user has write permissions for templates.
 	 *
-	 * @param array $ids
+	 * @param array  $templateids
+	 * @param string $error
 	 *
 	 * @return bool
 	 */
-	public function isWritable(array $ids) {
-		if (!is_array($ids)) {
-			return false;
+	private function checkPermissions(array $templateids, $error) {
+		if ($templateids) {
+			$templateids = array_unique($templateids);
+
+			$count = $this->get([
+				'countOutput' => true,
+				'templateids' => $templateids,
+				'editable' => true
+			]);
+
+			if ($count != count($templateids)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS, $error);
+			}
 		}
-		if (empty($ids)) {
-			return true;
-		}
-
-		$ids = array_unique($ids);
-
-		$count = $this->get([
-			'templateids' => $ids,
-			'editable' => true,
-			'countOutput' => true
-		]);
-
-		return (count($ids) == $count);
 	}
 
 	protected function addRelatedObjects(array $options, array $result) {
@@ -1129,10 +1149,9 @@ class CTemplate extends CHostGeneral {
 				]);
 				$templates = zbx_toHash($templates, 'templateid');
 				foreach ($result as $templateid => $template) {
-					if (isset($templates[$templateid]))
-						$result[$templateid]['templates'] = $templates[$templateid]['rowscount'];
-					else
-						$result[$templateid]['templates'] = 0;
+					$result[$templateid]['templates'] = array_key_exists($templateid, $templates)
+						? $templates[$templateid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1159,10 +1178,9 @@ class CTemplate extends CHostGeneral {
 				]);
 				$hosts = zbx_toHash($hosts, 'templateid');
 				foreach ($result as $templateid => $template) {
-					if (isset($hosts[$templateid]))
-						$result[$templateid]['hosts'] = $hosts[$templateid]['rowscount'];
-					else
-						$result[$templateid]['hosts'] = 0;
+					$result[$templateid]['hosts'] = array_key_exists($templateid, $hosts)
+						? $hosts[$templateid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1197,10 +1215,9 @@ class CTemplate extends CHostGeneral {
 				]);
 				$screens = zbx_toHash($screens, 'templateid');
 				foreach ($result as $templateid => $template) {
-					if (isset($screens[$templateid]))
-						$result[$templateid]['screens'] = $screens[$templateid]['rowscount'];
-					else
-						$result[$templateid]['screens'] = 0;
+					$result[$templateid]['screens'] = array_key_exists($templateid, $screens)
+						? $screens[$templateid]['rowscount']
+						: '0';
 				}
 			}
 		}
