@@ -24,11 +24,11 @@
 #define ZBX_STR(str)	ZBX_STR2(str)
 
 #define APPLICATION_NAME	"Zabbix Agent"
-#define ZABBIX_REVDATE		"29 July 2019"
+#define ZABBIX_REVDATE		"26 August 2019"
 #define ZABBIX_VERSION_MAJOR	4
 #define ZABBIX_VERSION_MINOR	0
-#define ZABBIX_VERSION_PATCH	11
-#define ZABBIX_VERSION_REVISION	53bb6bc0f0
+#define ZABBIX_VERSION_PATCH	12
+#define ZABBIX_VERSION_REVISION	fb4010108b
 #ifdef _WINDOWS
 #	define ZABBIX_VERSION_RC_NUM	1400
 #endif
