@@ -58,10 +58,12 @@ const char	*get_signal_name(int sig)
 		case SIGSEGV:	return "SIGSEGV";
 		case SIGBUS:	return "SIGBUS";
 		case SIGQUIT:	return "SIGQUIT";
+		case SIGABRT:	return "SIGABRT";
 		case SIGINT:	return "SIGINT";
 		case SIGTERM:	return "SIGTERM";
 		case SIGPIPE:	return "SIGPIPE";
 		case SIGUSR1:	return "SIGUSR1";
+		case SIGUSR2:	return "SIGUSR2";
 		default:	return "unknown";
 	}
 }
