@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -44,25 +44,32 @@ static int	get_fs_size_stat(const char *fs, zbx_uint64_t *total, zbx_uint64_t *f
 		return SYSINFO_RET_FAIL;
 	}
 
-	if (total)
+	/* Available space could be negative (top bit set) if we hit disk space */
+	/* reserved for non-privileged users. Treat it as 0.                    */
+	if (0 != ZBX_IS_TOP_BIT_SET(s.f_bavail))
+		s.f_bavail = 0;
+
+	if (NULL != total)
 		*total = (zbx_uint64_t)s.f_blocks * s.ZBX_BSIZE;
-	if (free)
+
+	if (NULL != free)
 		*free = (zbx_uint64_t)s.f_bavail * s.ZBX_BSIZE;
-	if (used)
+
+	if (NULL != used)
 		*used = (zbx_uint64_t)(s.f_blocks - s.f_bfree) * s.ZBX_BSIZE;
-	if (pfree)
+
+	if (NULL != pfree)
 	{
 		if (0 != s.f_blocks - s.f_bfree + s.f_bavail)
-			*pfree = (double)(100.0 * s.f_bavail) /
-					(s.f_blocks - s.f_bfree + s.f_bavail);
+			*pfree = (double)(100.0 * s.f_bavail) / (s.f_blocks - s.f_bfree + s.f_bavail);
 		else
 			*pfree = 0;
 	}
-	if (pused)
+
+	if (NULL != pused)
 	{
 		if (0 != s.f_blocks - s.f_bfree + s.f_bavail)
-			*pused = 100.0 - (double)(100.0 * s.f_bavail) /
-					(s.f_blocks - s.f_bfree + s.f_bavail);
+			*pused = 100.0 - (double)(100.0 * s.f_bavail) / (s.f_blocks - s.f_bfree + s.f_bavail);
 		else
 			*pused = 0;
 	}
@@ -116,7 +123,6 @@ static int	VFS_FS_TOTAL(const char *fs, AGENT_RESULT *result)
 	SET_UI64_RESULT(result, value);
 
 	return SYSINFO_RET_OK;
-
 }
 
 static int	VFS_FS_PFREE(const char *fs, AGENT_RESULT *result)
@@ -151,10 +157,9 @@ static int	VFS_FS_PUSED(const char *fs, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-int	VFS_FS_SIZE(AGENT_REQUEST *request, AGENT_RESULT *result)
+static int	vfs_fs_size(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	char	*fsname, *mode;
-	int	ret;
 
 	if (2 < request->nparam)
 	{
@@ -171,23 +176,25 @@ int	VFS_FS_SIZE(AGENT_REQUEST *request, AGENT_RESULT *result)
 		return SYSINFO_RET_FAIL;
 	}
 
-	if (NULL == mode || '\0' == *mode || 0 == strcmp(mode, "total"))
-		ret = VFS_FS_TOTAL(fsname, result);
-	else if (0 == strcmp(mode, "free"))
-		ret = VFS_FS_FREE(fsname, result);
-	else if (0 == strcmp(mode, "used"))
-		ret = VFS_FS_USED(fsname, result);
-	else if (0 == strcmp(mode, "pfree"))
-		ret = VFS_FS_PFREE(fsname, result);
-	else if (0 == strcmp(mode, "pused"))
-		ret = VFS_FS_PUSED(fsname, result);
-	else
-	{
-		SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid second parameter."));
-		return SYSINFO_RET_FAIL;
-	}
+	if (NULL == mode || '\0' == *mode || 0 == strcmp(mode, "total"))	/* default parameter */
+		return VFS_FS_TOTAL(fsname, result);
+	if (0 == strcmp(mode, "free"))
+		return VFS_FS_FREE(fsname, result);
+	if (0 == strcmp(mode, "pfree"))
+		return VFS_FS_PFREE(fsname, result);
+	if (0 == strcmp(mode, "used"))
+		return VFS_FS_USED(fsname, result);
+	if (0 == strcmp(mode, "pused"))
+		return VFS_FS_PUSED(fsname, result);
 
-	return ret;
+	SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid second parameter."));
+
+	return SYSINFO_RET_FAIL;
+}
+
+int	VFS_FS_SIZE(AGENT_REQUEST *request, AGENT_RESULT *result)
+{
+	return zbx_execute_threaded_metric(vfs_fs_size, request, result);
 }
 
 int	VFS_FS_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result)
